<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\ClassOfAccount;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Http\Request;
use App\Http\Requests\StoreOtherAccountMasterRequest;
use App\Http\Requests\UpdateOtherAccountMasterRequest;
use App\Permission;
use App\BranchPermission;
use App\AssignedPermission;
use App\Branches;

class AdminOtherAccountController extends Controller
{
    use SoftDeletes;

    public function index()
    {
        // $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();

        //  // Retrieve permission titles from Permission table
        //  $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        //  // Filter permission titles using hasBranchPermission function
        //  $filteredPermissionTitles = [];
        //  $branchId = session()->get('selected_branch');
        //  foreach ($permissionTitles as $permissionTitle) {
        //      if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
        //          $filteredPermissionTitles[] = $permissionTitle;
        //      }
        //  }

        //  // Retrieve AdminOtherAccount whose name_of_organisation matches filtered permission titles
        //  $otherAccounts = AdminOtherAccount::where('branch_id', $branchId)
        //      ->whereIn('account_name', $filteredPermissionTitles)
        //      ->orderBy('id', 'desc')
        //      ->get();

        $permissionTitles = Permission::where('branch_id', session()->get('selected_branch'))->pluck('name')->toArray();

        $filteredPermissionTitles = [];
        $branchId = session()->get('selected_branch');
        foreach ($permissionTitles as $permissionTitle) {
            if (Branches::hasBranchPermission($branchId, $permissionTitle)) {
                $filteredPermissionTitles[] = $permissionTitle;
            }
        }

        $otherAccounts = AdminOtherAccount::where('branch_id', $branchId)
            ->orderBy('id', 'desc')
            ->get();

        foreach ($otherAccounts as $otherAccount) {
            $otherAccount->account_name = str_replace(' ', '-', $otherAccount->account_name);
        }

        $filteredExpenses = $otherAccounts->filter(function ($customer) use ($filteredPermissionTitles) {
            return in_array($customer->account_name, $filteredPermissionTitles);
        });

        return view('crusher.otheraccount_master.index', compact('otherAccounts'));
    }

    public function create()
    {
        $classOfAccount = ClassOfAccount::all();
        return view('crusher.otheraccount_master.create', compact('classOfAccount'));
    }
    public function store(StoreOtherAccountMasterRequest $request)
    {
        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            AdminOtherAccount::create($validatedData);

            // use App\Permission;
            // use App\BranchPermission;
            // use App\AssignedPermission;

            // $name = $request->account_name;
            $name = str_replace(' ', '-', $request->account_name);
            $selectedBranchId = session('selected_branch');

            $permission = Permission::create([
                'name' => $name,
                'guard_name' => 'web',
                'type' => 'Ledger Other Account Master',
                'branch_id' => $selectedBranchId,
            ]);

            $BranchPermission = BranchPermission::create([
                'branch_id' => $permission->branch_id,
                'permission_id' => $permission->id,
            ]);

            if (auth()->check() && auth()->user()->role == 'subadmin') {
                $subadminId = auth()->user()->id;

                AssignedPermission::create([
                    'subadmin_id' => $subadminId,
                    'branch_permission_id' => $BranchPermission->id
                ]);
            } else {

            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.other_accounts.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($otherAccount)
    {
        $classOfAccount = ClassOfAccount::all();
        $otherAccount = AdminOtherAccount::find($otherAccount);
        return view('crusher.otheraccount_master.edit', compact('otherAccount','classOfAccount'));
    }

    function update(UpdateOtherAccountMasterRequest $request, $otherAccount)
    {
        try {
            $validatedData = $request->validate($request->rules());
            $validatedData['branch_id'] = session()->get('selected_branch');

            AdminOtherAccount::find($otherAccount)->update($validatedData);

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.other_accounts.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($otherAccount)
    {
        $otherAccount = AdminOtherAccount::find($otherAccount);
        return view('crusher.otheraccount_master.view', compact('otherAccount'));
    }

    public function softDelete($otherAccount)
    {
        $otherAccount = AdminOtherAccount::findOrFail($otherAccount);
        $otherAccount->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.other_accounts.index');
    }
}
