<?php

namespace App\Http\Controllers\Transaction;
use App\Http\Controllers\Controller;
use App\ReceiptVoucher;
use App\ReceiptVoucherItem;
use Illuminate\Http\Request;

use Auth;
use DB;
use Storage;
use Session;
use App\Supplier;
use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use App\AccountMasterTrack;
use App\ClassOfAccount;
use App\Currency;
use App\SaleInvoice;
use App\Services\Utils;
use App\Services\Slug;


class ReceiptVoucherController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('receipt-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $receipt_vouchers = ReceiptVoucher::withTrashed()->with('account')->where('location_id', $location_id)->orderBy('created_at','DESC')->paginate(100);
        return view('user.transactions.receipt_voucher.index', compact('receipt_vouchers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('receipt-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $latest_voucher = ReceiptVoucher::where('location_id', $location_id)->withTrashed()->latest()->first();
        if($latest_voucher){
            $invoice_no = $latest_voucher-> invoice_no + 1;
        }else{
            $invoice_no = 1;
        }

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $cash_class = ClassOfAccount::Where('name', 'Cash in Hand')->first();
        $bank_class = ClassOfAccount::Where('name', 'Bank Account')->first();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        return view('user.transactions.receipt_voucher.create',compact('currency','account_masters', 'invoice_no', 'cash_accounts', 'bank_accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store_old(Request $request)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $receipt_voucher = new ReceiptVoucher();
            $receipt_voucher->location_id = $location_id;
            $receipt_voucher->invoice_no = $request->invoice_no;
            $receipt_voucher->date = $request->date;
            $receipt_voucher->account_id = $request->selected_account_id;
            $receipt_voucher->amount = $request->amount;
            $receipt_voucher->remaining_amount = str_replace("-", "", $request->remaining_amount);
            $receipt_voucher->payment_method = $request->payment_method;
            $receipt_voucher->cheaque_no = $request->cheaque_no;
            $receipt_voucher->cheaque_date = $request->cheaque_date;
            $receipt_voucher->bank_name = $request->bank_name;
            $receipt_voucher->amount_in_words = 'amount_in_words';
            $receipt_voucher->narrations = $request->narrations;
            $receipt_voucher->save();



            foreach ($request->selected_payment_type as $key => $payment_type) {
                $receipt_voucher_item = new ReceiptVoucherItem();
                $receipt_voucher_item-> receipt_voucher_id = $receipt_voucher-> id;
                $receipt_voucher_item-> payment_type = $request->selected_payment_type[$key];
                $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
                if ($request->selected_payment_type[$key] == "billwise") {
                    $receipt_voucher_item->invoice_id = $request->select_invoice_id[$key];
                    $receipt_voucher_item->paid_amount = (float)$request->select_invoice_value[$key] - (float)$remaining;
                }
                if ($request->selected_payment_type[$key] == "on_account") {
                    $receipt_voucher_item->invoice_id = "";
                    $receipt_voucher_item->paid_amount = $remaining;
                }
                $receipt_voucher_item-> save();

            }


            $account = new AccountMastersAccountsDetail();
            $account->account_master_id = $request->selected_account_id;
            $account->debit = 0;
            $account->credit = $request->amount;
            $account->date = $request->date;
            $account->voucher_type = "receipt";
            $account->voucher_no = $request->invoice_no;
            $account->save();

            foreach ($request->select_invoice_id as $key => $invoice_id) {
                if ($request->select_invoice_id[$key] != "") {
                    $sales_invoice = SaleInvoice::Find($request->select_invoice_id[$key]);

                    $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
                    $paid = (float)$request->select_invoice_value[$key] - (float)$remaining;
                    $sales_invoice->paid_amount = (float)$sales_invoice->paid_amount + (float)$paid;
                    $sales_invoice->remaining_amount = $remaining;
                    $sales_invoice->save();
                }
            }


            DB::commit();
            Session::flash('alert-success','Receipt Voucher created successfully');
            return redirect('/transactions/receipt-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    public function store(Request $request)
    {
        // dd($request->all());
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $receipt_voucher = new ReceiptVoucher();
            $receipt_voucher->location_id = $location_id;
            $receipt_voucher->invoice_no = $request->invoice_no;
            $receipt_voucher->date = $request->date;
            $receipt_voucher->cash_account_id = $request->cash_accounts;
            $receipt_voucher->bank_account_id = $request->bank_accounts;
            // $receipt_voucher->amount = $request->amount;
            // $receipt_voucher->remaining_amount = str_replace("-", "", $request->remaining_amount);
            $receipt_voucher->payment_method = $request->payment_method;
            $receipt_voucher->cheaque_no = $request->cheaque_no;
            $receipt_voucher->cheaque_date = $request->cheaque_date;
            $receipt_voucher->bank_name = $request->bank_name;
            $receipt_voucher->amount_in_words = $request->amount_in_words;
            $receipt_voucher->net_total = $request->net_total;
            $receipt_voucher->narrations = $request->narrations;
            $receipt_voucher->save();



            // foreach ($request->selected_payment_type as $key => $payment_type) {
            //     $receipt_voucher_item = new ReceiptVoucherItem();
            //     $receipt_voucher_item-> receipt_voucher_id = $receipt_voucher-> id;
            //     $receipt_voucher_item-> payment_type = $request->selected_payment_type[$key];
            //     $remaining = str_replace("-", "", $request->invoice_remaining[$key]);
            //     if ($request->selected_payment_type[$key] == "billwise") {
            //         $receipt_voucher_item->invoice_id = $request->select_invoice_id[$key];
            //         $receipt_voucher_item->paid_amount = (float)$request->select_invoice_value[$key] - (float)$remaining;
            //     }
            //     if ($request->selected_payment_type[$key] == "on_account") {
            //         $receipt_voucher_item->invoice_id = "";
            //         $receipt_voucher_item->paid_amount = $remaining;
            //     }
            //     $receipt_voucher_item-> save();

            // }


            // $account = new AccountMastersAccountsDetail();
            // $account->account_master_id = $request->selected_account_id;
            // $account->debit = 0;
            // $account->credit = $request->amount;
            // $account->date = $request->date;
            // $account->voucher_type = "receipt";
            // $account->voucher_no = $request->invoice_no;
            // $account->save();

            foreach ($request->payment_type as $key => $payment_type) {
                $receipt_voucher_item = new ReceiptVoucherItem();
                $receipt_voucher_item-> receipt_voucher_id = $receipt_voucher-> id;
                $receipt_voucher_item-> payment_type = $request->payment_type[$key];
                $receipt_voucher_item-> amount = $request->amount[$key];
                $receipt_voucher_item-> account_id = $request->account_id[$key];
                if ($request->payment_type[$key] == "billwise") {
                    $receipt_voucher_item->invoice_id = $request->select_invoice[$key];
                }
                if ($request->payment_type[$key] == "on_account") {
                    $receipt_voucher_item->invoice_id = "";
                }
                $receipt_voucher_item-> save();

                $account = new AccountMastersAccountsDetail();
                $account->account_master_id = $request->account_id[$key];
                $account->credit = $request->amount[$key];
                $account->debit = 0;
                $account->date = $request->date;
                $account->voucher_type = "receipt";
                $account->voucher_no = $request->invoice_no;
                $account->save();

                if ($request->payment_method == "bank") {
                    $account_bank = new AccountMastersAccountsDetail();
                    $account_bank->account_master_id = $request->bank_accounts;
                    $account_bank->credit = 0;
                    $account_bank->debit = $request->amount[$key];
                    $account_bank->date = $request->date;
                    $account_bank->voucher_type = "receipt";
                    $account_bank->voucher_no = $request->invoice_no;
                    $account_bank->save();
                }elseif ($request->payment_method == "cash") {
                    $account_cash = new AccountMastersAccountsDetail();
                    $account_cash->account_master_id = $request->cash_accounts;
                    $account_cash->credit = 0;
                    $account_cash->debit = $request->amount[$key];
                    $account_cash->date = $request->date;
                    $account_cash->voucher_type = "receipt";
                    $account_cash->voucher_no = $request->invoice_no;
                    $account_cash->save();
                }

            }

            foreach ($request->select_invoice as $key => $invoice_id) {
                if ($request->select_invoice[$key] != "") {
                    $sales_invoice = SaleInvoice::Find($request->select_invoice[$key]);
                    $sales_invoice->paid_amount = (float)$sales_invoice->paid_amount + (float)$request->amount[$key] ;
                    $sales_invoice->remaining_amount = (float)$sales_invoice->remaining_amount - (float)$request->amount[$key];
                    $sales_invoice->save();
                }
            }


            DB::commit();
            Session::flash('alert-success','Receipt Voucher created successfully');
            return redirect('/transactions/receipt-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\ReceiptVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function show(ReceiptVoucher $paymentVoucher)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\ReceiptVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('edit-receipt-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $receipt_voucher = ReceiptVoucher::with('items')->where('id', $id)->first();

        $account_masters = AccountMasterTrack::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        $cash_class = ClassOfAccount::Where('name', 'Cash in Hand')->first();
        $bank_class = ClassOfAccount::Where('name', 'Bank Account')->first();
        $cash_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $cash_class->id)->get();
        $bank_accounts = AccountMaster::Where('location_id', $location_id)->Where('class_of_account_id', $bank_class->id)->get();
        $currency = Currency::Where('location_id', $location_id)->first();
        return view('user.transactions.receipt_voucher.edit', compact('currency','receipt_voucher','account_masters','cash_accounts', 'bank_accounts'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\ReceiptVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function update_old(Request $request, $id)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $receipt_voucher = ReceiptVoucher::where('id', $id)->first();
            $receipt_voucher->location_id = $location_id;
            $receipt_voucher->invoice_no = $request->invoice_no;
            $receipt_voucher->date = $request->date;
            $receipt_voucher->account_id = $request->account_id;
            $receipt_voucher->total_amount = $request->total_amount;
            $receipt_voucher->amount_in_words = 'amount_in_words';
            $receipt_voucher->narrations = $request->narrations;
            $receipt_voucher->save();

            ReceiptVoucherItem::where('receipt_voucher_id', $id)->forcedelete();

            foreach ($request->payment_method as $key => $payment_method) {
                $receipt_voucher_item = new ReceiptVoucherItem();
                $receipt_voucher_item-> receipt_voucher_id = $receipt_voucher-> id;
                $receipt_voucher_item-> payment_method = $request->payment_method[$key];
                $receipt_voucher_item-> cheaque_no = $request->cheaque_no[$key];
                $receipt_voucher_item-> cheaque_date = $request->cheaque_date[$key];
                $receipt_voucher_item-> bank_name = $request->bank_name[$key];
                $receipt_voucher_item-> amount = $request->amount[$key];
                $receipt_voucher_item-> save();
            }

            DB::commit();
            Session::flash('alert-success','Receipt Voucher updated successfully');
            return redirect('/transactions/receipt-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }


    public function update(Request $request, $id)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $receipt_voucher = ReceiptVoucher::where('id', $id)->first();
            $previous_cash_account = $receipt_voucher->cash_account_id;
            $previous_bank_account = $receipt_voucher->bank_account_id;
            $receipt_voucher->location_id = $location_id;
            $receipt_voucher->invoice_no = $request->invoice_no;
            $receipt_voucher->date = $request->date;
            $receipt_voucher->cash_account_id = $request->cash_accounts;
            $receipt_voucher->bank_account_id = $request->bank_accounts;
            $receipt_voucher->payment_method = $request->payment_method;
            $receipt_voucher->cheaque_no = $request->cheaque_no;
            $receipt_voucher->cheaque_date = $request->cheaque_date;
            $receipt_voucher->bank_name = $request->bank_name;
            $receipt_voucher->amount_in_words = $request->amount_in_words;
            $receipt_voucher->net_total = $request->net_total;
            $receipt_voucher->narrations = $request->narrations;
            $receipt_voucher->save();

            $receipt_item = ReceiptVoucherItem::where('receipt_voucher_id', $id)->get();


            foreach ($receipt_item as $item) {

                $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $item->account_id)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
                $account_detail->forcedelete();
                if ($previous_cash_account) {
                    $cash_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_cash_account)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
                    $cash_account_detail->forcedelete();
                }
                if ($previous_bank_account) {
                    $bank_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_bank_account)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
                    $bank_account_detail->forcedelete();
                }
                if ($item->payment_type == "billwise") {
                    $sale_invoice = SaleInvoice::Where('id', $item->invoice_id)->Where('account_id', $item->account_id)->first();
                    $sale_invoice->paid_amount = (float)$sale_invoice->paid_amount - (float)$item->amount;
                    $sale_invoice->remaining_amount = (float)$sale_invoice->remaining_amount + (float)$item->amount;
                    $sale_invoice->save();
                }
                $item->forcedelete();
            }

            foreach ($request->payment_type as $key => $payment_type) {
                $receipt_voucher_item = new ReceiptVoucherItem();
                $receipt_voucher_item-> receipt_voucher_id = $receipt_voucher-> id;
                $receipt_voucher_item-> payment_type = $request->payment_type[$key];
                $receipt_voucher_item-> amount = $request->amount[$key];
                $receipt_voucher_item-> account_id = $request->account_id[$key];
                if ($request->payment_type[$key] == "billwise") {
                    $receipt_voucher_item->invoice_id = $request->select_invoice[$key];
                }
                if ($request->payment_type[$key] == "on_account") {
                    $receipt_voucher_item->invoice_id = "";
                }
                $receipt_voucher_item-> save();

                $account = new AccountMastersAccountsDetail();
                $account->account_master_id = $request->account_id[$key];
                $account->credit = $request->amount[$key];
                $account->debit = 0;
                $account->date = $request->date;
                $account->voucher_type = "receipt";
                $account->voucher_no = $request->invoice_no;
                $account->save();

                if ($request->payment_method == "bank") {
                    $account_bank = new AccountMastersAccountsDetail();
                    $account_bank->account_master_id = $request->bank_accounts;
                    $account_bank->credit = 0;
                    $account_bank->debit = $request->amount[$key];
                    $account_bank->date = $request->date;
                    $account_bank->voucher_type = "receipt";
                    $account_bank->voucher_no = $request->invoice_no;
                    $account_bank->save();
                }elseif ($request->payment_method == "cash") {
                    $account_cash = new AccountMastersAccountsDetail();
                    $account_cash->account_master_id = $request->cash_accounts;
                    $account_cash->credit = 0;
                    $account_cash->debit = $request->amount[$key];
                    $account_cash->date = $request->date;
                    $account_cash->voucher_type = "receipt";
                    $account_cash->voucher_no = $request->invoice_no;
                    $account_cash->save();
                }

            }

            foreach ($request->select_invoice as $key => $invoice_id) {
                if ($request->select_invoice[$key] != "") {
                    $sales_invoice = SaleInvoice::Find($request->select_invoice[$key]);
                    $sales_invoice->paid_amount = (float)$sales_invoice->paid_amount + (float)$request->amount[$key] ;
                    $sales_invoice->remaining_amount = (float)$sales_invoice->remaining_amount - (float)$request->amount[$key];
                    $sales_invoice->save();
                }
            }

            DB::commit();
            Session::flash('alert-success','Receipt Voucher updated successfully');
            return redirect('/transactions/receipt-vouchers');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\ReceiptVoucher  $paymentVoucher
     * @return \Illuminate\Http\Response
     */
    public function destroy(ReceiptVoucher $paymentVoucher)
    {
        //
    }

    public function softdelete($id)
    {
        $user = Auth::user();
        if (!$user->can('cancel-receipt-voucher')) {
            abort(403, 'Unauthorized action.');
        }

        $receipt_voucher = ReceiptVoucher::Find($id);
        $previous_cash_account = $receipt_voucher->cash_account_id;
        $previous_bank_account = $receipt_voucher->bank_account_id;
        $receipt_item = ReceiptVoucherItem::where('receipt_voucher_id', $id)->get();

        foreach ($receipt_item as $item) {

            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $item->account_id)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
            $account_detail->delete();
            if ($previous_cash_account) {
                $cash_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_cash_account)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
                $cash_account_detail->delete();
            }
            if ($previous_bank_account) {
                $bank_account_detail = AccountMastersAccountsDetail::Where('account_master_id', $previous_bank_account)->where('voucher_type', "receipt")->where('voucher_no', $receipt_voucher->invoice_no)->first();
                $bank_account_detail->delete();
            }
            if ($item->payment_type == "billwise") {
                $sale_invoice = SaleInvoice::Where('id', $item->invoice_id)->Where('account_id', $item->account_id)->first();
                $sale_invoice->paid_amount = (float)$sale_invoice->paid_amount - (float)$item->amount;
                $sale_invoice->remaining_amount = (float)$sale_invoice->remaining_amount + (float)$item->amount;
                $sale_invoice->save();
            }
            $item->delete();
        }


        $receipt_voucher->delete();


        Session::flash('alert-success','Receipt Vouchers Cancelled successfully');
        return redirect('/transactions/receipt-vouchers');
    }
}
