<?php

namespace App\Http\Controllers\Master\AccountMaster;

use App\AccountMaster;
use App\AccountMastersAccountsDetail;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;

use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
// use App\Customer;
use App\AllCountries;
use App\ClassOfAccount;
use App\User;
use App\Location;
use App\LocationPermission;
use App\AccountMasterTrack;
use App\Permission;
use App\BranchPermission;
class CustomerController extends Controller
{

    protected $utils;

    public function __construct(Utils $utils)
    {
        $this->utils = $utils;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('customers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $customers = AccountMaster::Where('location_id', $location_id)->Where('account_master_type', "Customer")->orderBy('created_at','DESC')->paginate(100);
        $count = [];
        foreach ($customers as $customer) {
            $account_detail = AccountMastersAccountsDetail::Where('account_master_id', $customer->id)->where('voucher_type', '!=' ,"opening balance")->get();
            $count[] = count($account_detail);
        }
        return view('user.masters.account_masters.customers.index', compact('customers', 'count'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('customers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $all_countries = AllCountries::all();
        return view('user.masters.account_masters.customers.create', compact('all_countries', 'class_of_accounts', 'users'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request);
        $location_id = $this->getLocationId();
        $customer = AccountMaster::Where('name_of_organisation', $request->name_of_organisation)->Where('account_master_type', "Customer")->where('location_id', $location_id)->first();
        if ($customer) {
            Session::flash('alert-danger', 'This Customer Already Exists');
            return redirect()->back();
        }
        else
        // dd($request->all());
        // dd($location_id);
        $customer = new AccountMaster();
        $customer->location_id = $location_id;
        $customer->account_master_type = "Customer";
        if ($request->transactions_password != "") {
            $customer->transactions_password = Hash::make($request->transactions_password);
        }
        $customer->opening_balance = $request->opening_balance;
        $customer->debit_credit = $request->debit_credit;
        if ($request->report_password != "") {
            $customer->report_password = Hash::make($request->report_password);
        }
        $customer->date = $request->date;
        $customer->name_of_organisation = $request->name_of_organisation;
        $customer->commerical_registration_number = $request->commerical_registration_number;
        $customer->contact_person_name = $request->contact_person_name;
        $customer->office_phone = $request->office_phone;
        $customer->mobile = $request->mobile;
        $customer->whatsapp = $request->whatsapp;
        $customer->email = $request->email;
        $customer->country_id = $request->country_id;
        $customer->city = $request->city;
        $customer->address = $request->address;
        $customer->class_of_account_id = $request->class_of_account_id;
        // $customer->show_user = $request->show_user;
        $customer->maintain_account = $request->maintain_account;
        $customer->vat_number = $request->vat_number;
        $customer->trn_number = $request->trn_number;
        $customer->tin_number = $request->tin_number;
        $customer->doc_name_1 = $request->doc_name_1;
        if ($request->hasFile('doc_file_1')) {
            $file = $request->file('doc_file_1');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_1 = $filePath;
        }
        $customer->doc_expiry_1 = $request->doc_expiry_1;

        $customer->doc_name_2 = $request->doc_name_2;
        if ($request->hasFile('doc_file_2')) {
            $file = $request->file('doc_file_2');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_2 = $filePath;
        }
        $customer->doc_expiry_2 = $request->doc_expiry_2;

        $customer->doc_name_3 = $request->doc_name_3;
        if ($request->hasFile('doc_file_3')) {
            $file = $request->file('doc_file_3');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_3 = $filePath;
        }
        $customer->doc_expiry_3 = $request->doc_expiry_3;

        $customer->doc_name_4 = $request->doc_name_4;
        if ($request->hasFile('doc_file_4')) {
            $file = $request->file('doc_file_4');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_4 = $filePath;
        }
        $customer->doc_expiry_4 = $request->doc_expiry_4;

        $customer->doc_name_5 = $request->doc_name_5;
        if ($request->hasFile('doc_file_5')) {
            $file = $request->file('doc_file_5');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_5 = $filePath;
        }
        $customer->doc_expiry_5 = $request->doc_expiry_5;

        $customer->doc_name_6 = $request->doc_name_6;
        if ($request->hasFile('doc_file_6')) {
            $file = $request->file('doc_file_6');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_6 = $filePath;
        }
        $customer->doc_expiry_6 = $request->doc_expiry_6;


        $customer->doc_name_7 = $request->doc_name_7;
        if ($request->hasFile('doc_file_7')) {
            $file = $request->file('doc_file_7');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_7 = $filePath;
        }
        $customer->doc_expiry_7 = $request->doc_expiry_7;

        $customer->authorised_sign_1_name = $request->authorised_sign_1_name;
        $customer->authorised_sign_1_civil_id = $request->authorised_sign_1_civil_id;
        $customer->authorised_sign_1_expiry_date = $request->authorised_sign_1_expiry_date;
        if ($request->hasFile('authorised_sign_1_sign_sample')) {
            $file = $request->file('authorised_sign_1_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_1_sign_sample = $filePath;
        }
        $customer->authorised_sign_1_phone = $request->authorised_sign_1_phone;

        $customer->authorised_sign_2_name = $request->authorised_sign_2_name;
        $customer->authorised_sign_2_civil_id = $request->authorised_sign_2_civil_id;
        $customer->authorised_sign_2_expiry_date = $request->authorised_sign_2_expiry_date;
        if ($request->hasFile('authorised_sign_2_sign_sample')) {
            $file = $request->file('authorised_sign_2_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_2_sign_sample = $filePath;
        }
        $customer->authorised_sign_2_phone = $request->authorised_sign_2_phone;

        $customer->authorised_sign_3_name = $request->authorised_sign_3_name;
        $customer->authorised_sign_3_civil_id = $request->authorised_sign_3_civil_id;
        $customer->authorised_sign_3_expiry_date = $request->authorised_sign_3_expiry_date;
        if ($request->hasFile('authorised_sign_3_sign_sample')) {
            $file = $request->file('authorised_sign_3_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_3_sign_sample = $filePath;
        }
        $customer->authorised_sign_3_phone = $request->authorised_sign_3_phone;

        $customer->description = $request->description;

        $customer->save();


        $current_customer_id = $customer->id;

        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $current_customer_id;
                $account_master_track->account_master_name = "Customer";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_customer_id;
            $account_master_track->account_master_name = "Customer";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $current_customer_id;
                $account_master_track->account_master_name = "Customer";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $current_customer_id;
            $account_master_track->account_master_name = "Customer";
            $account_master_track->save();
        }

        if ($request->opening_balance != "") {
            $account = new AccountMastersAccountsDetail;
            $account->account_master_id = $customer->id;
            if ($request->debit_credit == "debit") {
                $account->debit = $request->opening_balance;
                $account->credit = 0;
            }elseif ($request->debit_credit == "credit") {
                $account->debit = 0;
                $account->credit = $request->opening_balance;
            }
            $account->date = $request->date;
            $account->voucher_type = "opening balance";
            $account->save();
        }



        Session::flash('alert-success', 'Customer created successfully');
        return redirect('/masters/account-masters/customers');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Customer  $customer
     * @return \Illuminate\Http\Response
     */
    public function show(Customer $customer)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Customer  $customer
     * @return \Illuminate\Http\Response
     */
    public function edit($customer_id)
    {
        $user = Auth::user();
        if (!$user->can('customers-account-master')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $country = Location::where('id', $location_id)->first();
        $class_of_accounts = ClassOfAccount::all();
        $users = LocationPermission::with('users')->where('country_id', $country->country_id)->where('location_id', $location_id)->where('user_id', '!=', $user->id)->get();
        $account_master_track = AccountMasterTrack::Where('account_master_id', $customer_id)
                                                    ->Where('account_master_name', "Customer")->pluck('user_id');
        $all_countries = AllCountries::all();
        $customer = AccountMaster::Where('id', $customer_id)
                                   ->Where('account_master_type', "Customer")->first();
        return view('user.masters.account_masters.customers.edit', compact('customer', 'all_countries', 'class_of_accounts', 'users', 'account_master_track'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Customer  $customer
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $customer_id)
    {
        // dd($request->all());
        $customer = AccountMaster::Where('id', $customer_id)
                                    ->Where('account_master_type', "Customer")->first();
        $previous_opening_balance = (float)$customer->opening_balance;
        $current_opening_balance = (float)$request->opening_balance;
        if ($request->transactions_password != "") {
            $customer->transactions_password = Hash::make($request->transactions_password) ? Hash::make($request->transactions_password) : $customer->transactions_password;
        }
        $customer->opening_balance = $request->opening_balance;
        $customer->debit_credit = $request->debit_credit;
        if ($request->report_password != "") {
            $customer->report_password = Hash::make($request->report_password) ? Hash::make($request->report_password) : $customer->report_password;
        }
        $customer->date = $request->date;
        $customer->name_of_organisation = $request->name_of_organisation;
        $customer->commerical_registration_number = $request->commerical_registration_number;
        $customer->contact_person_name = $request->contact_person_name;
        $customer->office_phone = $request->office_phone;
        $customer->mobile = $request->mobile;
        $customer->whatsapp = $request->whatsapp;
        $customer->email = $request->email;
        $customer->country_id = $request->country_id;
        $customer->city = $request->city;
        $customer->address = $request->address;
        $customer->class_of_account_id = $request->class_of_account_id;
        // $customer->show_user = $request->show_user;
        $customer->maintain_account = $request->maintain_account;
        $customer->vat_number = $request->vat_number;
        $customer->trn_number = $request->trn_number;
        $customer->tin_number = $request->tin_number;
        $customer->doc_name_1 = $request->doc_name_1;
        if ($request->hasFile('doc_file_1')) {
            $file = $request->file('doc_file_1');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_1 = $filePath;
        }
        $customer->doc_expiry_1 = $request->doc_expiry_1;

        $customer->doc_name_2 = $request->doc_name_2;
        if ($request->hasFile('doc_file_2')) {
            $file = $request->file('doc_file_2');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_2 = $filePath;
        }
        $customer->doc_expiry_2 = $request->doc_expiry_2;

        $customer->doc_name_3 = $request->doc_name_3;
        if ($request->hasFile('doc_file_3')) {
            $file = $request->file('doc_file_3');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_3 = $filePath;
        }
        $customer->doc_expiry_3 = $request->doc_expiry_3;

        $customer->doc_name_4 = $request->doc_name_4;
        if ($request->hasFile('doc_file_4')) {
            $file = $request->file('doc_file_4');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_4 = $filePath;
        }
        $customer->doc_expiry_4 = $request->doc_expiry_4;

        $customer->doc_name_5 = $request->doc_name_5;
        if ($request->hasFile('doc_file_5')) {
            $file = $request->file('doc_file_5');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_5 = $filePath;
        }
        $customer->doc_expiry_5 = $request->doc_expiry_5;

        $customer->doc_name_6 = $request->doc_name_6;
        if ($request->hasFile('doc_file_6')) {
            $file = $request->file('doc_file_6');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_6 = $filePath;
        }
        $customer->doc_expiry_6 = $request->doc_expiry_6;


        $customer->doc_name_7 = $request->doc_name_7;
        if ($request->hasFile('doc_file_7')) {
            $file = $request->file('doc_file_7');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_doc/'), $filePath);
            $customer-> doc_file_7 = $filePath;
        }
        $customer->doc_expiry_7 = $request->doc_expiry_7;

        $customer->authorised_sign_1_name = $request->authorised_sign_1_name;
        $customer->authorised_sign_1_civil_id = $request->authorised_sign_1_civil_id;
        $customer->authorised_sign_1_expiry_date = $request->authorised_sign_1_expiry_date;
        if ($request->hasFile('authorised_sign_1_sign_sample')) {
            $file = $request->file('authorised_sign_1_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_1_sign_sample = $filePath;
        }
        $customer->authorised_sign_1_phone = $request->authorised_sign_1_phone;

        $customer->authorised_sign_2_name = $request->authorised_sign_2_name;
        $customer->authorised_sign_2_civil_id = $request->authorised_sign_2_civil_id;
        $customer->authorised_sign_2_expiry_date = $request->authorised_sign_2_expiry_date;
        if ($request->hasFile('authorised_sign_2_sign_sample')) {
            $file = $request->file('authorised_sign_2_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_2_sign_sample = $filePath;
        }
        $customer->authorised_sign_2_phone = $request->authorised_sign_2_phone;

        $customer->authorised_sign_3_name = $request->authorised_sign_3_name;
        $customer->authorised_sign_3_civil_id = $request->authorised_sign_3_civil_id;
        $customer->authorised_sign_3_expiry_date = $request->authorised_sign_3_expiry_date;
        if ($request->hasFile('authorised_sign_3_sign_sample')) {
            $file = $request->file('authorised_sign_3_sign_sample');
            $filePath = time() . '-' . $this->utils->randstr() . '.' . $file->getClientOriginalExtension();
            $file->move(public_path('/user_files/img/customer_authorised_sign_sample/'), $filePath);
            $customer-> authorised_sign_3_sign_sample = $filePath;
        }
        $customer->authorised_sign_3_phone = $request->authorised_sign_3_phone;

        $customer->description = $request->description;
        $customer->save();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $customer_id)
                                                    ->Where('account_master_name', "Customer")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }
        if(isset($request->show_user)){
            foreach ($request->show_user as $key => $request->show_user) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $request->show_user;
                $account_master_track->account_master_id = $customer_id;
                $account_master_track->account_master_name = "Customer";
                $account_master_track->save();
            }
        }

        if(Auth::user()->hasRole('super-admin'))
        {
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $customer_id;
            $account_master_track->account_master_name = "Customer";
            $account_master_track->save();
        }else{
            $admins = User::role('super-admin')->get();
            foreach ($admins as $key => $admin) {
                $account_master_track = new AccountMasterTrack();
                $account_master_track->user_id = $admin->id;
                $account_master_track->account_master_id = $customer_id;
                $account_master_track->account_master_name = "Customer";
                $account_master_track->save();
            }
            $account_master_track = new AccountMasterTrack();
            $account_master_track->user_id = Auth::user()->id;
            $account_master_track->account_master_id = $customer_id;
            $account_master_track->account_master_name = "Customer";
            $account_master_track->save();
        }

        // if ($request->opening_balance != "") {
        //     $account = AccountMastersAccountsDetail::Where('account_master_id', $request->account_id)->first();

        //     if ($account != "") {
        //         if ($previous_opening_balance > $current_opening_balance) {
        //             $diff = $previous_opening_balance - $current_opening_balance;
        //             $account->credit =  (float)$account->credit - (float)$diff;
        //         }
        //         elseif ($previous_opening_balance < $current_opening_balance) {
        //             $diff = $current_opening_balance - $previous_opening_balance;
        //             $account->credit = (float)$account->credit + (float)$diff;
        //         }

        //         $account->save();

        //     }else{
        //         $account = new AccountMastersAccountsDetail();
        //         $account->account_master_id = $customer->id;
        //         $account->debit = 0;
        //         $account->credit = $request->opening_balance;
        //         $account->save();
        //     }

        // }

        Session::flash('alert-success', 'Customer updated successfully');
        return redirect('/masters/account-masters/customers');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Customer  $customer
     * @return \Illuminate\Http\Response
     */
    public function destroy($customer_id)
    {
        $customer = AccountMaster::Where('id', $customer_id)->Where('account_master_type', "Customer")->first();
        $customer->delete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $customer_id)
                                                    ->Where('account_master_name', "Customer")->get();
        foreach ($account_master_track as $item) {
            $item->delete();
        }

        Session::flash('alert-success', 'Customer deleted successfully');
        return redirect('/masters/account-masters/customers');
    }

    public function softdelete($customer_id)
    {
        $customer = AccountMaster::Where('id', $customer_id)->Where('account_master_type', "Customer")->first();
        $customer->forceDelete();

        $account_master_track = AccountMasterTrack::Where('account_master_id', $customer_id)
                                                    ->Where('account_master_name', "Customer")->get();
        foreach ($account_master_track as $item) {
            $item->forceDelete();
        }

        Session::flash('alert-success', 'Customer deleted successfully');
        return redirect('/masters/account-masters/customers');
    }

}
