<?php

namespace App\Http\Controllers\Report;

use App\AccountMastersAccountsDetail;
use App\ClassOfAccount;
use App\Currency;
use App\Http\Controllers\Controller;
use App\PurchaseInvoice;
use App\PurchaseInvoiceItem;
use App\PurchaseReturnVoucher;
use App\PurchaseReturnVoucherItem;
use App\SaleInvoice;
use App\SaleInvoiceItem;
use App\SalesReturnVoucher;
use App\SalesReturnVoucherItem;
use App\ItemGroup;
use App\Services\Utils;
use Illuminate\Http\Request;
use PDF;
use Auth;

class ProfitLossReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function display()
    {
        $user = Auth::user();
        if (!$user->can('profit-and-loss-report')) {
            abort(403, 'Unauthorized action.');
        }

        return view('user.reports.profit_loss.select_date');
    }

    public function index(Request $request)
    {
        $user = Auth::user();
        if (!$user->can('profit-and-loss-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $gross_total_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $sales_invoices = $gross_total_sales_invoices - $discount_sales_invoices;
        $gross_total_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $sales_return_vouchers = $gross_total_sales_return_vouchers - $discount_sales_return_vouchers;
        $sales_accounts = $sales_invoices - $sales_return_vouchers;
        $sales_accounts_decimal = number_format($sales_accounts, $decimal, '.', '');


        $gross_total_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $purchase_invoices = $gross_total_purchase_invoices - $discount_purchase_invoices;
        $gross_total_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $purchase_return_vouchers = $gross_total_purchase_return_vouchers - $discount_purchase_return_vouchers;
        $purchase_accounts = $purchase_invoices - $purchase_return_vouchers;
        $purchase_accounts_decimal = number_format($purchase_accounts, $decimal, '.', '');
        // dd($purchase_accounts);

        $groups = ItemGroup::with('item_names.purchase_sales_transfer_stock_tracks')->with('item_names.purchase_invoice')->Where('location_id', $location_id)->get();

        $group_total_amount = 0;
        $purchase_invoice = 0;

        foreach ($groups as $group){
            foreach($group->item_names as $item_name){
                foreach ($item_name->purchase_sales_transfer_stock_tracks as $key => $purchase_sales_transfer_stock_track){
                    $purchase_invoice = $item_name->purchase_invoice->where('item_name_id', $purchase_sales_transfer_stock_track->item_name_id)->where('imei_serial_no', $purchase_sales_transfer_stock_track->imei_serial_no)->first();
                    if($purchase_invoice){
                        $rate_per_quantity = $purchase_invoice->rate_per_quantity;
                    }else{
                        $rate_per_quantity = 0;
                    }
                    $group_total_amount = $group_total_amount + ($purchase_sales_transfer_stock_track->quantity * $rate_per_quantity);
                }
            }
        }

        $closing_amount =  $group_total_amount;
        $closing_amount_decimal =  number_format($group_total_amount, $decimal, '.', '');
        // dd($closing_amount);

        $opening_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');

        $opening_inward_amount = $opening_purchase_invoices + $opening_sales_return_vouchers;
        $opening_outward_amount = $opening_sales_invoices + $opening_purchase_return_vouchers;
        $opening_amount = $opening_inward_amount - $opening_outward_amount;
        $opening_amount_decimal =  number_format($opening_amount, $decimal, '.', '');
        // dd($opening_amount);

        $cost_of_sales = ($opening_amount + $purchase_accounts) - $closing_amount;
        $cost_of_sales_decimal = number_format($cost_of_sales, $decimal, '.', '');
        // dd($cost_of_sales);

        $gross_profit_loss = $sales_accounts - $cost_of_sales;
        $gross_profit_loss_decimal = number_format($gross_profit_loss, $decimal, '.', '');
        // dd($gross_profit_loss_decimal);

        $expenses_accounts_closing_balance = $this->closingBalanceClassOfAccounts('Expenses', $location_id, $from_date, $to_date);

        $expense_total = 0;
        foreach ($expenses_accounts_closing_balance as $closing) {
            $expense_total = $expense_total + $closing['closing_balance'];
        }

        $expense_total = $expense_total + $discount_sales_invoices;
        $expense_total_decimal = number_format($expense_total, $decimal, '.', '');

        $discount_sales_invoices_decimal = number_format($discount_sales_invoices, $decimal, '.', '');

        $income_accounts_closing_balance = $this->closingBalanceClassOfAccounts('Income', $location_id, $from_date, $to_date);

        $income_total = 0;
        foreach ($income_accounts_closing_balance as $closing) {
            $income_total = $income_total + $closing['closing_balance'];
        }

        $income_total = $income_total + $discount_purchase_invoices;
        $income_total_decimal = number_format($income_total, $decimal, '.', '');


        $discount_purchase_invoices_decimal = number_format($discount_purchase_invoices, $decimal, '.', '');

        $net_profit_loss = ($gross_profit_loss - $expense_total) + $income_total;
        $net_profit_loss_decimal =  number_format($net_profit_loss, $decimal, '.', '');

        return view('user.reports.profit_loss.index', compact('from_date', 'to_date', 'sales_accounts_decimal', 'purchase_accounts_decimal', 'opening_amount_decimal', 'closing_amount_decimal', 'cost_of_sales_decimal', 'gross_profit_loss_decimal', 'expenses_accounts_closing_balance', 'discount_sales_invoices_decimal', 'income_total_decimal', 'expense_total_decimal', 'net_profit_loss_decimal'));
    }

    public function pdfView($from_date, $to_date){

        $user = Auth::user();
        if (!$user->can('profit-and-loss-report')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = Utils::getLocation();

        $decimal = 0;

        $currency = Currency::Where('location_id', $location_id)->first();
        if ($currency) {
            $decimal = $currency->decimal_currency;
        }

        $gross_total_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $sales_invoices = $gross_total_sales_invoices - $discount_sales_invoices;
        $gross_total_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $sales_return_vouchers = $gross_total_sales_return_vouchers - $discount_sales_return_vouchers;
        $sales_accounts = $sales_invoices - $sales_return_vouchers;
        $sales_accounts_decimal = number_format($sales_accounts, $decimal, '.', '');


        $gross_total_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $purchase_invoices = $gross_total_purchase_invoices - $discount_purchase_invoices;
        $gross_total_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('gross_total');
        $discount_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('discount');
        $purchase_return_vouchers = $gross_total_purchase_return_vouchers - $discount_purchase_return_vouchers;
        $purchase_accounts = $purchase_invoices - $purchase_return_vouchers;
        $purchase_accounts_decimal = number_format($purchase_accounts, $decimal, '.', '');
        // dd($purchase_accounts);

        $groups = ItemGroup::with('item_names.purchase_sales_transfer_stock_tracks')->with('item_names.purchase_invoice')->Where('location_id', $location_id)->get();

        $group_total_amount = 0;

        foreach ($groups as $group){
            foreach($group->item_names as $item_name){
                foreach ($item_name->purchase_sales_transfer_stock_tracks as $key => $purchase_sales_transfer_stock_track){
                    $purchase_invoice = $item_name->purchase_invoice->where('item_name_id', $purchase_sales_transfer_stock_track->item_name_id)->where('imei_serial_no', $purchase_sales_transfer_stock_track->imei_serial_no)->first();
                    if($purchase_invoice){
                        $rate_per_quantity = $purchase_invoice->rate_per_quantity;
                    }else{
                        $rate_per_quantity = 0;
                    }
                    $group_total_amount = $group_total_amount + ($purchase_sales_transfer_stock_track->quantity * $rate_per_quantity);
                }
            }
        }

        $closing_amount =  $group_total_amount;
        $closing_amount_decimal =  number_format($group_total_amount, $decimal, '.', '');
        // dd($closing_amount);

        $opening_sales_invoices = SaleInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_purchase_invoices = PurchaseInvoice::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_sales_return_vouchers = SalesReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');
        $opening_purchase_return_vouchers = PurchaseReturnVoucher::with('account')->Where('location_id', $location_id)->Where('date', '<' , $from_date)->sum('gross_total');

        $opening_inward_amount = $opening_purchase_invoices + $opening_sales_return_vouchers;
        $opening_outward_amount = $opening_sales_invoices + $opening_purchase_return_vouchers;
        $opening_amount = $opening_inward_amount - $opening_outward_amount;
        $opening_amount_decimal =  number_format($opening_amount, $decimal, '.', '');
        // dd($opening_amount);

        $cost_of_sales = ($opening_amount + $purchase_accounts) - $closing_amount;
        $cost_of_sales_decimal = number_format($cost_of_sales, $decimal, '.', '');
        // dd($cost_of_sales);

        $gross_profit_loss = $sales_accounts - $cost_of_sales;
        $gross_profit_loss_decimal = number_format($gross_profit_loss, $decimal, '.', '');
        // dd($gross_profit_loss_decimal);

        $expenses_accounts_closing_balance = $this->closingBalanceClassOfAccounts('Expenses', $location_id, $from_date, $to_date);

        $expense_total = 0;
        foreach ($expenses_accounts_closing_balance as $closing) {
            $expense_total = $expense_total + $closing['closing_balance'];
        }

        $expense_total = $expense_total + $discount_sales_invoices;
        $expense_total_decimal = number_format($expense_total, $decimal, '.', '');

        $discount_sales_invoices_decimal = number_format($discount_sales_invoices, $decimal, '.', '');

        $income_accounts_closing_balance = $this->closingBalanceClassOfAccounts('Income', $location_id, $from_date, $to_date);

        $income_total = 0;
        foreach ($income_accounts_closing_balance as $closing) {
            $income_total = $income_total + $closing['closing_balance'];
        }

        $income_total = $income_total + $discount_purchase_invoices;
        $income_total_decimal = number_format($income_total, $decimal, '.', '');


        $discount_purchase_invoices_decimal = number_format($discount_purchase_invoices, $decimal, '.', '');

        $net_profit_loss = ($gross_profit_loss - $expense_total) + $income_total;
        $net_profit_loss_decimal =  number_format($net_profit_loss, $decimal, '.', '');

        $pdf = PDF::loadView('user.reports.profit_loss.list', compact('from_date', 'to_date', 'sales_accounts_decimal', 'purchase_accounts_decimal', 'opening_amount_decimal', 'closing_amount_decimal', 'cost_of_sales_decimal', 'gross_profit_loss_decimal', 'expenses_accounts_closing_balance', 'discount_sales_invoices_decimal', 'income_total_decimal', 'expense_total_decimal', 'net_profit_loss_decimal'));
        return $pdf->stream();
    }

    public function closingBalanceClassOfAccounts($account_type, $location_id, $from_date, $to_date){
        $trial_balance_closing_balance = [];
        $class_of_account = ClassOfAccount::with(['account_master' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->where('name', $account_type)->first();
        if ($class_of_account) {

            foreach ($class_of_account->account_master as $account_master){
                $debit = 0;
                $credit = 0;
                $opening_balance = 0;
                $closing_balance = 0;
                $debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('debit');
                $credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->where('voucher_type', '!=', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->sum('credit');
                $first_account = AccountMastersAccountsDetail::with('account_master')->Where('account_master_id', $account_master->id)->first();
                if($first_account){
                    $with_opening_balance_debit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('debit');
                    $with_opening_balance_credit = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('date', '>=' , $first_account->date)->Where('date', '<=' , date('Y-m-d', strtotime('-1 day', strtotime($from_date))))->sum('credit');
                }else{
                    $with_opening_balance_debit = 0;
                    $with_opening_balance_credit = 0;
                }
                $voucher_type_opening_balance = AccountMastersAccountsDetail::Where('account_master_id', $account_master->id)->Where('voucher_type', 'opening balance')->Where('date', '>=' , $from_date)->Where('date', '<=' , $to_date)->first();
                if($voucher_type_opening_balance){
                    if($voucher_type_opening_balance-> debit > 0){
                        $opening_balance = $voucher_type_opening_balance-> debit;
                    }else{
                        $opening_balance = $voucher_type_opening_balance->credit;
                    }
                }else {
                    if($account_master-> account_master_type == 'Supplier'){
                        $opening_balance = ($with_opening_balance_credit) - $with_opening_balance_debit;
                    }else{
                        $opening_balance = ($with_opening_balance_debit) - $with_opening_balance_credit;
                    }
                }
                if($account_master-> account_master_type == 'Supplier'){
                    $closing_balance = ($opening_balance + $credit) - $debit;
                }else{
                    $closing_balance = ($opening_balance + $debit) - $credit;
                }
                $trial_balance_closing_balance[] = [
                    "ledger" => $account_master,
                    "closing_balance" => $closing_balance,
                ];
            }
        }
        return $trial_balance_closing_balance;
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
