<?php

namespace App\Http\Controllers;

use App\AssignedPermission;
use App\Branches;
use App\BranchPermission;
use App\Http\Requests\StoreSubadminRequest;
use App\Subadmin;
use App\Customer;
use App\Supplier;
use App\StaffAndSalesman;
use Illuminate\Support\Facades\DB;
use App\Expense;
use App\AdminOtherAccount;

use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
class SubadminController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $subadmins = User::where(['admin_id'=>auth()->id(), 'status'=>1])->get();
        return view('crusher.subadmin.crusher-users', compact('subadmins'));
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $branches = Branches::with('company','permissions.permission')->whereHas('company', function($q) {$q->where('user_id', session()->get('uv_user_id'));})->get();
        // Get the ID of the currently selected branch from the session
        $selectedBranchId = session('selected_branch');

        // Find the selected branch within the fetched branches
        $selectedBranch = $branches->first(function ($branch) use ($selectedBranchId) {
            return $branch->id == $selectedBranchId;
        });
        $permissions=[];
        if (isset(request()->branch_ids)) {
            $permissions = BranchPermission::with('permission', 'branch')
            ->whereIn('branch_id', request()->branch_ids)
            ->get();
        }

         $customers = Customer::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get();


         $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
         ->select('*', 'name_of_organisation as account_name')
         ->get();

        $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
         ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
         ->get();

         $expenses = Expense::where('branch_id', session()->get('selected_branch'))
         ->get();

         $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
         ->get();

        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        // $accountNames = $accounts->pluck('account_name');

        $accountNames = $accounts->pluck('account_name')->map(function ($name) {
            return str_replace(' ', '-', $name);
        });

        // dd($accountNames);

        return view('crusher.subadmin.crusher-add-users', compact('selectedBranch', 'branches', 'permissions','accountNames'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    function store(StoreSubadminRequest $request)
    {
        try {
            $validatedData = $request->validated();
            $permissions = $validatedData['permissions']??[];
            $validatedData['password'] = bcrypt($validatedData['password']);
            $validatedData['admin_id'] = auth()->id();
            $validatedData['role'] = 'subadmin';
            // $validatedData['agreed'] = $request->has('agreed') ? 1 : 0;
            $validatedData['agreed'] = $request->has('agreed') ? 0 : 1;

            unset($validatedData['permissions']);

            // dd($permissions);

            $subadmin = User::create($validatedData);

            if (!empty($permissions)) {
                foreach ($permissions as $permission) {
                    AssignedPermission::create(['subadmin_id'=>$subadmin->id, 'branch_permission_id'=>$permission]);
                }
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.subadmins.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }


    public function show(User $subadmin)
    {
        $subadmin->load('permissions');
        $branches = Branches::with('company','permissions.permission')->whereHas('company', function($q) {$q->where('user_id', session()->get('uv_user_id'));})->get();
        // Get the ID of the currently selected branch from the session
        $selectedBranchId = session('selected_branch');

        // Find the selected branch within the fetched branches
        $selectedBranch = $branches->first(function ($branch) use ($selectedBranchId) {
            return $branch->id == $selectedBranchId;
        });
        $permissions=[];
        if (isset(request()->branch_ids)) {
            $permissions = BranchPermission::with('permission', 'branch')
            ->whereIn('branch_id', request()->branch_ids)
            ->get();
        }

        return view('crusher.subadmin.crusher-view-users', compact('selectedBranch', 'branches', 'permissions', 'subadmin'));
    }


    public function edit(User $subadmin)
    {
        $subadmin->load('permissions');
        $branches = Branches::with('company','permissions.permission')->whereHas('company', function($q) {$q->where('user_id', session()->get('uv_user_id'));})->get();
        // Get the ID of the currently selected branch from the session
        $selectedBranchId = session('selected_branch');

        // Find the selected branch within the fetched branches
        $selectedBranch = $branches->first(function ($branch) use ($selectedBranchId) {
            return $branch->id == $selectedBranchId;
        });
        $permissions=[];
        if (isset(request()->branch_ids)) {
            $permissions = BranchPermission::with('permission', 'branch')
            ->whereIn('branch_id', request()->branch_ids)
            ->get();
        }

        return view('crusher.subadmin.crusher-edit-users', compact('selectedBranch', 'branches', 'permissions', 'subadmin'));
    }

    public function update(StoreSubadminRequest $request, $subadmin)
{
    try {
        $validatedData = $request->validated();
        $permissions = $validatedData['permissions'] ?? [];
        // $validatedData['agreed'] = $request->has('agreed') ? 1 : 0;
        $validatedData['agreed'] = $request->has('agreed') ? 0 : 1;

        if (!empty($validatedData['password'])) {
            $validatedData['password'] = bcrypt($validatedData['password']);
        } else {
            unset($validatedData['password']);
        }

        unset($validatedData['permissions']);

        $subadmin = User::find($subadmin);
        $subadmin->update($validatedData);

        if (!empty($permissions)) {
            $subadmin->permissions()->delete();
            foreach ($permissions as $permission) {
                AssignedPermission::create(['subadmin_id' => $subadmin->id, 'branch_permission_id' => $permission]);
            }
        }

        flash()->addSuccess('Record Updated');
        return redirect()->route('crusher.subadmins.index');
    } catch (\Throwable $th) {
        flash()->addError($th->getMessage());
        return redirect()->back()->withInput();
    }
}


    public function destroy($subadmin)
    {
        User::find($subadmin)->update(['status'=>0]);
        flash()->addSuccess('Success');
        return redirect()->back();
    }

    public function changepassword() {
        return view('crusher.crusher-changepass');
    }

    public function updatepass(Request $request)
{
    $request->validate([
        'old_password' => 'required',
        'password' => 'required|string|min:8|confirmed',
    ]);

    $user = Auth::user();

    if (!Hash::check($request->old_password, $user->password)) {
        return redirect()->route('password.change')->with('error', 'The old password is incorrect.');
    }

    $user->password = Hash::make($request->password);
    $user->save();

    return redirect()->back()->with('success', 'Password changed successfully.');
}
}
