<?php

namespace App\Services;
use App\User;
use App\Country;
use App\Location;
use App\Customer;
use App\WareHouse;
use App\ItemGroup;
use App\ItemName;

class Slug
{
    /**
     * @param $title
     * @param int $id
     * @return string
     * @throws \Exception
     */
    public function createSlug($type, $title, $id = 0)
    {
        // Normalize the title
        $slug = str_slug($title);
        // Get any that could possibly be related.
        // This cuts the queries down by doing it once.
        $allSlugs = $this->getRelatedSlugs($type, $slug, $id);
        // If we haven't used it before then we are all good.
        if (! $allSlugs->contains('slug', $slug)){
            return $slug;
        }
        // Just append numbers like a savage until we find not used.
        for ($i = 1; $i <= 10000; $i++) {
            $newSlug = $slug.'-'.$i;
            if (! $allSlugs->contains('slug', $newSlug)) {
                return $newSlug;
            }
        }
        throw new \Exception('Can not create a unique slug');
    }
    protected function getRelatedSlugs($type, $slug, $id = 0)
    {
        if($type == 'country') {

            return Country::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }else if($type == 'location') {

            return Location::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }else if($type == 'customer') {

            return Customer::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }else if($type == 'warehouse') {

            return WareHouse::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }else if($type == 'item_group') {

            return ItemGroup::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }else if($type == 'item_name') {

            return ItemName::select('slug')->where('slug', 'like', $slug.'%')
                        ->where('id', '<>', $id)
                        ->get();

        }
    }

}
